@@ -89,7 +89,11 @@ class DeepMILModule(LightningModule):
 
         # Model components
         self.encoder = encoder_params.get_encoder(outputs_folder)
-        self.aggregation_fn, self.num_pooling = pooling_params.get_pooling_layer(self.encoder.num_encoding)
+        self.projector = encoder_params.get_projection_layer(self.encoder.num_encoding)
+        # If projection is enabled, the number of encoding dimensions is the projection dimension otherwise it is the
+        # number of encoding dimensions of the encoder.
+        num_encoding = encoder_params.projection_dim if encoder_params.projection_dim > 0 else self.encoder.num_encoding
+        self.aggregation_fn, self.num_pooling = pooling_params.get_pooling_layer(num_encoding)
         self.classifier_fn = classifier_params.get_classifier(self.num_pooling, self.n_classes)
         self.activation_fn = self.get_activation()
 
@@ -262,7 +266,8 @@ class DeepMILModule(LightningModule):
 
     def forward(self, instances: Tensor) -> Tuple[Tensor, Tensor]:  # type: ignore
         instance_features = self.get_instance_features(instances)
-        attentions, bag_features = self.get_attentions_and_bag_features(instance_features)
+        projected_features = self.projector(instance_features)
+        attentions, bag_features = self.get_attentions_and_bag_features(projected_features)
         bag_logit = self.get_bag_logit(bag_features)
         return bag_logit, attentions
 
